module cde_totalder; % CDIFF package, definition of total derivatives

algebraic;

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%
%    * Redistributions of source code must retain the relevant copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in the
%      documentation and/or other materials provided with the distribution.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNERS OR
% CONTRIBUTORS
% BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%

% % *****************************************************************
% Author and maintainer: Raffaele Vitolo
% Dipartimento di Matematica, Universita' del Salento (Lecce, Italy)
% email: raffaele.vitolo@unisalento.it
% web: http://poincare.unisalento.it/vitolo
%
% Version and Date:  v. 1.0, October 2014.
%
% ===============================================================

%-----------------------------------------------------------------------------%
% Procedures for the definition of total derivatives
%-----------------------------------------------------------------------------%

procedure set_svf(comp,i,j,val);
   % This procedure is needed because REDUCE refuses
   % to define or evaluate an operator in an iteration by a direct call.
   comp(i,j):=val;

procedure ev_svf(el,ell);
   % This procedure is needed because REDUCE refuses
   % to define or evaluate an operator in an iteration by a direct call.
   el(ell);

procedure coeff_totder(par,var,i);
   % this procedure finds the coefficient u^a_{\sigma i}
   % of \pd{}{u^a_\sigma}, where u^a_\sigma is an element of a list
   % of variables `variables'. A similar procedure is introduced
   % for odd coordinates.
   begin scalar var_mind,vartemp,miptemp,var_i;
      var_mind:=idtomind(par,var);
      vartemp:=first var_mind;
      miptemp:=second asfirst(second var_mind,all_mind_table);
      if miptemp neq {} then
      <<
         if par=0 then
            var_i:=first aslast({vartemp,part(miptemp,i)},i2m_jetspace)
         else
            var_i:=first aslast({vartemp,part(miptemp,i)},i2m_jetspace_odd)
      >>
      else var_i:=letop;
      return var_i
   end;

procedure initialize_td(td,param_der,princ_der,param_ext,princ_ext);
   % defines the components of total derivatives on the free jet space
   % with respect to independent variables using the previous procedure set_svf
   % The arguments of the procedure are identifiers of total derivatives
   % and the list of derivative coordinates.
   % Note that odd derivatives have to be entered in `ext' format.
   % Note that highest order derivatives have `letop' as a coefficient.
   % So, if letop appears in a computation one knows that
   % the computation must be redone with higher order operators.
   begin scalar n_ders,n_odd_ders,var_i,ovar_i,ovar_i_ext,ddtemp,
      	 dcon,dcon_odd;
      % component of the independent variable
      for i:=1:n_indep_var do
      <<
         ddtemp:=part(td,i);
   	 for j:=1:n_indep_var do
   	 <<
      	    if i=j then set_svf(ddtemp,0,j,1)
      	    else set_svf(ddtemp,0,j,0);
   	 >>;
      >>;
      % components of even variables
      dcon:={};
      n_param_der:=length(param_der);
      n_princ_der:=length(princ_der);
      for i:=1:n_indep_var do
      <<
         ddtemp:=part(td,i);
   	 for j:=1:n_param_der do
   	 <<
            var_i:=coeff_totder(0,part(param_der,j),i);
            if var_i member all_principal_der then
               dcon:={ddtemp,0,j+n_indep_var,var_i} . dcon;
            set_svf(ddtemp,0,j+n_indep_var,var_i)
	       %               write "coefficient of ",part(param_der,j),":";
	       %               write ddtemp,"(0,",j+n_indep_var,"):=",var_i;
   	 >>;
         for j:=1:n_princ_der do
   	 <<
            var_i:=coeff_totder(0,part(princ_der,j),i);
            set_svf(ddtemp,0,j+n_indep_var+n_param_der,var_i)
	       %               write "coefficient of ",part(ders,j),":";
	       %               write ddtemp,"(0,",j+n_indep_var,"):=",var_i;
   	 >>
      >>;
      % components of odd variables
      dcon_odd:={};
      n_param_ext:=length(param_ext);
      n_princ_ext:=length(princ_ext);
      for i:=1:n_indep_var do
      <<
         ddtemp:=part(td,i);
         for j:=1:n_param_ext do
   	 <<
            ovar_i:=
               coeff_totder(1,first aslast(part(param_ext,j),o2e_jetspace),i);
            if ovar_i neq letop then
            <<
               if ovar_i member all_principal_odd then
                  dcon_odd:={ddtemp,1,j,ovar_i} . dcon_odd;
               set_svf(ddtemp,1,j,second asfirst(ovar_i,o2e_jetspace))
            >>
            else set_svf(ddtemp,1,j,letop)
	       % write "coefficient of ",replace_extodd(part(odd_ders,j)),":";
	       % write ddtemp,"(1,",j+n_indep_var,"):=",ovar_i;
   	 >>;
   	 for j:=1:n_princ_ext do
   	 <<
            ovar_i:=
               coeff_totder(1,first aslast(part(princ_ext,j),o2e_jetspace),i);
            if ovar_i neq letop then
               set_svf(ddtemp,1,j+n_param_ext,
                  second asfirst(ovar_i,o2e_jetspace))
            else set_svf(ddtemp,1,j+n_param_ext,letop)
	       % write "coefficient of ",replace_extodd(part(odd_ders,j)),":";
	       % write ddtemp,"(1,",j+n_indep_var,"):=",ovar_i;
   	 >>
      >>;
      return {reverse(dcon),reverse(dcon_odd)}
   end;

procedure def_td(td_name,dd_name);
   % Define total derivative with name `td_var' in external coordinates
      for all arg_phi let td_name(arg_phi) =
         replace_oddext(dd_name(replace_extodd(arg_phi)));

procedure cde_total_derivatives();
begin scalar tempprinc;
   % Definition of the total derivatives vector fields,
   % together with the lists of all even and odd principal derivatives
   % which are components of parametric derivatives.
   %
   % Total derivatives identifiers (e.g, ddx, ddt,...)
   % NOTE: if you wish to change the prefix, change the variable (e)id_tot_der
   tot_der:=for i:=1:n_indep_var collect mkid(id_tot_der,part(indep_var,i));
%   etot_der:=for i:=1:n_indep_var collect mkid(eid_tot_der,part(indep_var,i));
   % Total derivatives: declaration.
   for i:=1:n_indep_var do
      super_vectorfield(part(tot_der,i),
         append(indep_var,append(all_parametric_der,all_principal_der)),
         all_ext);
   % Define total derivatives in external coordinates
%   for i:=1:n_indep_var do def_td(part(etot_der,i),part(tot_der,i));
   % Auxiliary variable `letop'. It appears
   % in computations where total_order is too low.
   for each el in indep_var do depend letop,el;
   % Definition of total derivatives.
   % The output format of initialize_td is a list of two elements.
   % The first element is a list of elements of the form
   % {ddtemp,0,j+n_indep_var,var_i}
   % The second element is a list of elements of the form
   % {ddtemp,1,j,ovar_i}
   primary_diffcon_tot:=initialize_td(tot_der,
      all_parametric_der,all_principal_der,
      all_parametric_ext,all_principal_ext);
   % In the following lists, initial principal derivatives are removed
   % because we do not need to compute their expression.
   % Such lists are said to be `primary differential consequences'
   tempprinc:=
      mkset(for each el in first(primary_diffcon_tot) collect last el);
   primary_diffcon_der:=diffset(tempprinc,principal_der);
   tempprinc:=
      mkset(for each el in second(primary_diffcon_tot) collect last el);
   primary_diffcon_odd:=diffset(tempprinc,principal_odd);
end;

endmodule;

end;
