% KdV equation: local Hamiltonian operators, verification of skew-adjointness
% and computation of Schouten bracket
% 2013-10-17
% Raffaele Vitolo

% Loading the interface to cdiff packages; used to generate all
% (even and odd) derivative coordinates,
% (even and odd) principal and parametric derivatives,
% and the restriction of total derivatives to the (even and odd) equation.
% Of course mostly odd part refers to tangent or cotangent covering.
in "cde.red"$

% Initialization of the jet environment of the differential equation.
indep_var:={x,t}$
dep_var:={u}$
odd_var:={p}$
deg_indep_var:={-1,-3}$
deg_dep_var:={2}$
deg_odd_var:={1}$
total_order:=10$
% names for output of the state of cde and results of computations
statename:="kdv_lho3_state.red"$
resname:="kdv_lho3_res.red"$

% Initialization of the differential equation.
% The system must be in passive orthonomic form;
% this also means that there will be no nontrivial integrability conditions
% between parametric derivatives.

% left-hand side of the differential equation
principal_der:={u_t}$
% right-hand side of the differential equation
de:={u*u_x+u_3x}$

% same construction for odd coordinates
principal_odd:={p_t}$
de_odd:={u*p_x+p_3x}$

% Calling cde's main routine
cde({indep_var,dep_var,odd_var,total_order},
   {{principal_der,de},{principal_odd,de_odd}})$
% Saving cde state in a file
save_cde_state(statename)$

% Define the two Hamiltonian operators for KdV
operator ham1;
for all psi1 let ham1(psi1)=ddx(psi1);
operator ham2;
for all psi2 let ham2(psi2)=(1/3)*u_x*psi2 + ddx(ddx(ddx(psi2)))
 + (2/3)*u*ddx(psi2);

% Convert the two Hamiltonian operators to the corresp. generating functions.
sym1:=conv_cdiff2genfun(ham1);
sym2:=conv_cdiff2genfun(ham2);

% Defines the previously computed Hamiltonian operators,
% the user may check that they are the same as sym1 and sym2
sym1_odd := {p_x};
sym2_odd := {(1/3)*p*u_x + p_3x + (2/3)*p_x*u};

% Converts the two operators to bivectors
biv1 := conv_genfun2biv(sym1_odd);
biv2 := conv_genfun2biv(sym2_odd);

% Checks that bivectors are skewadjoint
check_skewadjoint(biv1);
check_skewadjoint(biv2);

% Computes the Schouten bracket of the operators;
sb11 := schouten_bracket(biv1,biv1);
sb12 := schouten_bracket(biv1,biv2);
sb22 := schouten_bracket(biv2,biv2);

% Here we write results of the computation in a file.

off nat$
off echo$
out <<resname>>$
write sb11:=sb11;
write sb12:=sb12;
write sb22:=sb22;
write ";end;";
shut <<resname>>$
on echo$
on nat$

;end;

